"""
Common tools
"""
import time

from google.cloud import error_reporting

report = error_reporting.Client()


def report_exception(func):
    """Decorator for exceptions. Send error in Google Error Reporting and cancel exception."""

    def exception_wrapper(**kwargs):
        try:
            result = func(**kwargs)
            return result
        except Exception as exc:
            report.report(str(exc)[:2048])

    return exception_wrapper


def timeit(method):
    def wrapper(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        if 'log_time' in kw:
            name = kw.get('log_name', method.__name__.upper())
            kw['log_time'][name] = int((te - ts) * 1000)
        else:
            print('%r  %2.2f ms' % (method.__name__, (te - ts) * 1000))
        return result

    return wrapper



__all__ = ('log',)

import os


def log(method):
    def wrapper(*args, **kwargs):
        if os.getenv('IS_ENABLE_API_LOG'):
            if method.__name__ == '__init__':
                print(f'INIT: {args[0]}')
            elif method.__name__ == '_get':
                print(f'GET SERVICE {args[0]}')
            elif method.__name__ == '_flush_self_cache':
                print(f'FLUSH CACHE {args[0]}')
        result = method(*args, **kwargs)
        return result

    return wrapper



__all__ = ('Scoring1CPoint',)

from expressmoney.api import *


SERVICE_NAME = 'scoring'


class CreditScoring1CCreateContract(Contract):

    MALE = "0"
    FEMALE = "1"
    GENDER_CHOICES = (
        (MALE, "Male"),
        (FEMALE, "Female")
    )
    first_name = serializers.CharField(min_length=1, max_length=32)
    last_name = serializers.CharField(min_length=1, max_length=32)
    middle_name = serializers.CharField(min_length=1, max_length=32)
    gender = serializers.ChoiceField(choices=GENDER_CHOICES)
    birth_date = serializers.DateField()
    passport_serial = serializers.CharField(min_length=4, max_length=4)
    passport_number = serializers.CharField(min_length=6, max_length=6)
    passport_date = serializers.DateField()
    score = serializers.DecimalField(5, 4, min_value=0, max_value=1, read_only=True)


class CreditScoring1CReadContract(CreditScoring1CCreateContract):
    pagination = PaginationContract()
    id = serializers.IntegerField(min_value=1)
    score = serializers.DecimalField(5, 4, min_value=0, max_value=1)


# Endpoints ID
scoring_credit_scoring_1c_scoring_1c = ID(SERVICE_NAME, 'credit_scoring_1c', 'scoring_1c')


# Endpoints handlers
class Scoring1CPoint(ListPointMixin, CreatePointMixin, ContractPoint):
    _point_id = scoring_credit_scoring_1c_scoring_1c
    _create_contract = CreditScoring1CCreateContract
    _read_contract = CreditScoring1CReadContract



__all__ = ('BankCardPoint',)

from expressmoney.api import *


SERVICE_NAME = 'payments'


class BankCardCreateContract(Contract):
    M01 = "01"
    M02 = "02"
    M03 = "03"
    M04 = "04"
    M05 = "05"
    M06 = "06"
    M07 = "07"
    M08 = "08"
    M09 = "09"
    M10 = "10"
    M11 = "11"
    M12 = "12"

    MONTH_CHOICES = (
        (M01, '01'),
        (M02, '02'),
        (M03, '03'),
        (M04, '04'),
        (M05, '05'),
        (M06, '06'),
        (M07, '07'),
        (M08, '08'),
        (M09, '09'),
        (M10, '10'),
        (M11, '11'),
        (M12, '12'),
    )

    Y2020 = "2020"
    Y2021 = "2021"
    Y2022 = "2022"
    Y2023 = "2023"
    Y2024 = "2024"
    Y2025 = "2025"
    Y2026 = "2026"
    Y2027 = "2027"
    Y2028 = "2028"
    Y2029 = "2029"
    Y2030 = "2030"

    YEAR_CHOICES = (
        (Y2020, "2020"),
        (Y2021, "2021"),
        (Y2022, "2022"),
        (Y2023, "2023"),
        (Y2024, "2024"),
        (Y2025, "2025"),
        (Y2026, "2026"),
        (Y2027, "2027"),
        (Y2028, "2028"),
        (Y2029, "2029"),
        (Y2030, "2030"),
    )

    bin = serializers.CharField(max_length=6)
    number = serializers.CharField(max_length=4)
    expiry_month = serializers.ChoiceField(choices=MONTH_CHOICES)
    expiry_year = serializers.ChoiceField(choices=YEAR_CHOICES)
    ip = serializers.IPAddressField(allow_blank=True)
    cryptogram = serializers.CharField(max_length=1024, allow_blank=True)
    redirect_url = serializers.URLField(max_length=512, allow_blank=True)


class BankCardReadContract(BankCardCreateContract):
    PAYPAL = "PAYPAL"
    CLOUDPAYMENTS = "CLOUDPAYMENTS"
    GATEWAY_CHOICES = (
        (PAYPAL, "PayPal"),
        (CLOUDPAYMENTS, "CloudPayments")
    )
    pagination = PaginationContract()
    id = serializers.IntegerField(min_value=1)
    is_active = serializers.BooleanField(default=True)
    gateway = serializers.ChoiceField(choices=GATEWAY_CHOICES)


payments_bank_cards_bank_card = ID(SERVICE_NAME, 'bank_cards', 'bank_card')


class BankCardPoint(ListPointMixin, CreatePointMixin, ContractPoint):
    _point_id = payments_bank_cards_bank_card
    _create_contract = BankCardCreateContract
    _read_contract = BankCardReadContract


"""
from django.contrib.auth import get_user_model
User = get_user_model()
user = User.objects.get(id=1)
"""



__all__ = ('RequestPhotoVerificationPoint', 'RequestVideoVerificationPoint',
           'AutoVerificationProcessPoint', 'VideoVerificationProcessPoint', 'PhotoVerificationProcessPoint',
           )

from expressmoney.api import *
from expressmoney.viewflow import status

SERVICE = 'profiles'


class RequestPhotoVerificationCreateContract(Contract):
    file = serializers.CharField(max_length=256)


class RequestVideoVerificationCreateContract(Contract):
    TELEGRAM = 'TELEGRAM'
    WHATSAPP = 'WHATSAPP'

    MESSENGER_CHOICE = (
        (TELEGRAM, 'Telegram'),
        (WHATSAPP, 'WhatsApp'),
    )
    user_message = serializers.CharField(max_length=256, allow_blank=True)
    messenger = serializers.ChoiceField(choices=MESSENGER_CHOICE)


class AutoVerificationProcessReadContract(Contract):
    RESULT_CHOICES = (
        (status.ERROR, 'System error'),
        (status.FAILURE, 'Decline'),
        (status.SUCCESS, 'Verified'),
    )
    pagination = PaginationContract()
    id = serializers.IntegerField(min_value=1)
    status = serializers.CharField(max_length=50)
    profile = serializers.IntegerField(min_value=1)
    result = serializers.ChoiceField(choices=RESULT_CHOICES)


class VideoVerificationProcessReadContract(Contract):
    RESULT_CHOICES = (
        (status.ERROR, 'Incorrect phonenumber'),
        (status.IN_PROCESS, 'Recall'),
        (status.FAILURE, 'Decline'),
        (status.SUCCESS, 'Approved'),
    )

    WAIT_10M = 'WAIT_10M'
    WAIT_30M = 'WAIT_30M'
    WAIT_1H = 'WAIT_1H'
    WAIT_1D = 'WAIT_1D'

    RECALL_CHOICES = (
        (WAIT_10M, '10 minutes'),
        (WAIT_30M, '30 minutes'),
        (WAIT_1H, '1 hour'),
        (WAIT_1D, 'Tomorrow'),
    )

    pagination = PaginationContract()
    id = serializers.IntegerField(min_value=1)
    status = serializers.CharField(max_length=50)
    user_id = serializers.IntegerField(min_value=1)
    profile = serializers.IntegerField(min_value=1)
    result = serializers.ChoiceField(choices=RESULT_CHOICES)
    recall = serializers.ChoiceField(choices=RECALL_CHOICES, allow_blank=True)
    attempts = serializers.IntegerField(min_value=0)
    message = serializers.CharField(max_length=64, allow_blank=True)
    messenger = serializers.CharField(max_length=16, allow_blank=True)
    user_message = serializers.CharField(max_length=256, allow_blank=True)


class PhotoVerificationProcessReadContract(Contract):
    RESULT_CHOICES = (
        (status.IN_PROCESS, status.IN_PROCESS),
        (status.FAILURE, status.FAILURE),
        (status.SUCCESS, status.SUCCESS),
    )

    pagination = PaginationContract()
    id = serializers.IntegerField(min_value=1)
    status = serializers.CharField(max_length=50)

    user_id = serializers.IntegerField(min_value=1)
    profile = serializers.IntegerField(min_value=1)
    result = serializers.ChoiceField(choices=RESULT_CHOICES)
    file = serializers.CharField(max_length=256)
    message = serializers.CharField(max_length=64, allow_blank=True)


profiles_verification_request_photo_verification = ID(SERVICE, 'verification', 'request_photo_verification')
profiles_verification_request_video_verification = ID(SERVICE, 'verification', 'request_video_verification')
profiles_verification_auto_verification_process = ID(SERVICE, 'verification', 'auto_verification_process')
profiles_verification_video_verification_process = ID(SERVICE, 'verification', 'video_verification_process')
profiles_verification_photo_verification_process = ID(SERVICE, 'verification', 'photo_verification_process')


class RequestPhotoVerificationPoint(CreatePointMixin, ContractPoint):
    _point_id = profiles_verification_request_photo_verification
    _create_contract = RequestPhotoVerificationCreateContract


class RequestVideoVerificationPoint(CreatePointMixin, ContractPoint):
    _point_id = profiles_verification_request_video_verification
    _create_contract = RequestVideoVerificationCreateContract


class AutoVerificationProcessPoint(ListPointMixin, ContractPoint):
    _point_id = profiles_verification_auto_verification_process
    _read_contract = AutoVerificationProcessReadContract


class VideoVerificationProcessPoint(ListPointMixin, ContractPoint):
    _point_id = profiles_verification_video_verification_process
    _read_contract = VideoVerificationProcessReadContract


class PhotoVerificationProcessPoint(ListPointMixin, ContractPoint):
    _point_id = profiles_verification_photo_verification_process
    _read_contract = PhotoVerificationProcessReadContract



__all__ = ('UserPoint', 'SendPasswordPoint',)

from phonenumber_field.serializerfields import PhoneNumberField

from expressmoney.api import *

SERVICE_NAME = 'auth'


auth_user_send_password = ID(SERVICE_NAME, 'user', 'send_password')
auth_user = ID(SERVICE_NAME, 'user', None)


class UserCreateContract(Contract):
    username = PhoneNumberField()
    ip = serializers.IPAddressField()
    http_referer = serializers.URLField()


class SendPasswordCreateContract(Contract):
    username = PhoneNumberField()


class SendPasswordPoint(CreatePointMixin, ContractPoint):
    _point_id = auth_user_send_password
    _create_contract = SendPasswordCreateContract


class UserPoint(CreatePointMixin, ContractPoint):
    _point_id = auth_user
    _create_contract = UserCreateContract
